
/* Simon Sings OPEN  - Now ready for servo code (where marked: @TODO), to use operating a latch;
 *             ================================================================================
 * http://www.instructables.com/id/Improved-Simon-Says-Code/
 *
 "Sings" is the sound-enhanced version of "Fairly Simple Simon".
 No relation to the Singh family :)
 Original Breadboard diagram, video, etc. at http://bit.ly/simonsings
 
 After reset, Simon plays a bar from "Simple Simon says", and starts playing a game at level 4.
 
 Playing a game at level N:
 1) Simon picks a random sequence of N LED flashes.
 2) Simon waits for you to press any button when ready.
 3) Simon "says" the sequence. Memorize it.
 4) You should then repeat the sequence on the buttons.
    * If you're right, Simon plays the song, and you start a level N+1 game at step 1.
    * If you push a wrong button, Simon plays a sad tune, and you go back to step 2.
 
 cc-by-sa by @TheRealDod, Nov 23, 2010
 RRM Oct 2015: allowed for faster button inputs
 RRM Mar 2016  adapted Simon_Sings_OPEN version; prepared for use as a door opener
 */
#include "toneNotes.h"

const int OPEN_CNT = 5;
const int NLEDS = 4; 
const int SPEAKERPIN = 14;  // D14 is aka A0, have used pin 4, 8 ... 13 
const int LEDPINS[NLEDS] = {2,3,17,16}; // Need to be PWM pins ??
const int SWITCHPINS[NLEDS] = {5,6,7,8}; // 

const int SWITCHPRESSED = 0; // 1 or 0, for normally-open/closed switches
const int NOTES[NLEDS] = {NOTE_C4, NOTE_D4, NOTE_E4, NOTE_F4};

const int FADESTEPS = 8;
const int FADEINDURATION = 200;
const int FADEOUTDURATION = 150;
const int SEQDELAY = 50; // Millis between led flashes.
const int PAUSEB4SEQ = 500; // Millis before starting the sequence.
bool watchBtn=false;  // used to keep LED illumination activity from blocking Button activity

const int MINLEVEL = 4;
const int MAXLEVEL = 16;
int gameLevel;
int simonSez[MAXLEVEL]; // sequence of 0..NLEDS-1


// -- song-array note fields --
// Tone
const int NOTETONE = 0;
const int SILENCE = 0;
const int ENDOFSONG = -1;
// Duration
const int NOTEDURATION = 1;
const int SINGLEBEAT = 125; // Note duration (millis) is multiplied by this
const float PAUSEFACTOR=0.2; // relative length of silence after playing a note
// LED
const int NOTELED = 2;
const int ALLLEDS = -1;

int WINSONG[][3] = {
  {SILENCE,2,ALLLEDS}
  ,{NOTE_E4,1,2} ,{NOTE_E4,1,2} ,{NOTE_E4,1,2}
  ,{NOTE_F4,1,3} ,{NOTE_E4,1,2} ,{NOTE_D4,3,1}
  ,{NOTE_G4,1,ALLLEDS} ,{NOTE_G4,1,ALLLEDS} ,{NOTE_G4,1,ALLLEDS}
  ,{NOTE_A4,2,ALLLEDS} ,{NOTE_G4,5,ALLLEDS}
  ,{ENDOFSONG,ENDOFSONG,ENDOFSONG}
};

int LOSESONG[][3] = {
  {NOTE_B5,2,3},{NOTE_A5,2,2},{NOTE_GS5,2,1},{NOTE_G5,8,ALLLEDS},{ENDOFSONG,ENDOFSONG,ENDOFSONG}
};

// =============================================================
void setup() {
  // @TODO HERE DO SERVO SETUP
  // @TODO INITIALIZATION SERVO TO CLOSED (or NORMALLY DESIRED STATE)
  
  Serial.begin(9600);    // use serial
  // Analog in 0 should *not* be connected.
  // It's mama's little PRNG :)
  randomSeed(analogRead(0));
  
  pinMode(SPEAKERPIN,OUTPUT);
  noTone(SPEAKERPIN);
  gameLevel=MINLEVEL;
  for (byte l=0; l<NLEDS; l++) {
    pinMode(LEDPINS[l], OUTPUT);
    pinMode(SWITCHPINS[l], INPUT);
    digitalWrite(SWITCHPINS[l],HIGH);
  }

  beep(50); beep(50); beep(50);
  delay ( 1000 );

  // Visual feedback after reset. Also good as a cable check :)
  playWinSequence();
//  // initGameSequence as 1,2,3,4
//  for (int i=0; i<gameLevel; i++) {
//    simonSez[i]=i;
//  }
  initGameSequence(gameLevel);
}

void loop() {
  int done;
  done = 0;
  while (!done) {
    if (gameLevel == MINLEVEL) {  // wait for a key to start
      getSwitchStroke();
    }
    delay(PAUSEB4SEQ);
    playGameSequence(gameLevel);
    if (playerGuess(gameLevel)) {
//      playWinSequence();
      done = 1;
      if (gameLevel<MAXLEVEL) {
        gameLevel++;
      }
      extendSequence();
      if (gameLevel==OPEN_CNT+1) {
        // @TODO HERE ADD CODE FOR SERVO OPEN (or desired special state)
        Serial.println("\n Simon Says Open the Lock ");
        playWinSequence();
        delay(5000);  // allow time for person to acually Open the door
        // @TODO HERE ADD CODE FOR SERVO RETURN TO NORMAL STATE
        Serial.println(" Re-Close ");
        gameLevel = MINLEVEL;
        initGameSequence(gameLevel);
      }
    } 
    else {
      playLoseSequence();
      gameLevel = MINLEVEL;
      initGameSequence(gameLevel);
    }
  }
}

// ===========================================================
//             Local Utilities

void initGameSequence(int gameLevel) {
  // assertion: gameLevel<=MAXLEVEL
  for (int i=0; i<gameLevel; i++) {
    simonSez[i]=random(NLEDS);
  }
}


void extendSequence() {
    simonSez[gameLevel-1]=random(NLEDS);
}

void playGameSequence(int gameLevel) {
  for (int i=0; i<gameLevel; i++) {
    playLed(simonSez[i]); // Fade LED and play its note
  }
}

void fadeLed(int theLed,int val,int duration) {
  int fadeStep=256/FADESTEPS;
  int fadeDelay=duration/FADESTEPS;
  for (int i=0; i<256; i+=fadeStep) {
    if (theLed>=0) {
      analogWrite(LEDPINS[theLed],val?i:255-i);
    } 
    else { // ALLLEDS
      for (int j=0; j<NLEDS; j++) {
        analogWrite(LEDPINS[j],val?i:255-i);
      }
    }
    delay(fadeDelay);
    if (watchBtn && btnReased()) break; // to allow fast button input sequences
  }
  
  // force val (in case fadeStep doesn't divide 256)
  if (theLed>=0) {
    digitalWrite(LEDPINS[theLed],val); 
  }
  else {
    for (int j=0; j<NLEDS; j++) {
      digitalWrite(LEDPINS[j],val); 
    }
  }
}

void playLed(int theLed) { // Fade LED and play its note
  tone(SPEAKERPIN,NOTES[theLed]);
  fadeLed(theLed,HIGH,FADEINDURATION); // Fade in
  noTone(SPEAKERPIN);
  fadeLed(theLed,LOW,FADEOUTDURATION); // Fade out
}

int playerGuess(int gameLevel) {
  for (int i=0 ; i<gameLevel ; i++) {
    int guess=getSwitchStroke();
    //Serial.print(guess,DEC);
    //Serial.print(",");
    //Serial.println(simonSez[i]);
    if (guess!=simonSez[i]) {
      return 0;
    } 
    else {
      watchBtn = true;
      playLed(guess); // Fade LED and play its note
      watchBtn = false;
    }
  }
  return 1;
}

void playSong(int song[][3]) {
  for (int note=0; song[note][NOTETONE]!=ENDOFSONG; note++) {
    int theDuration=SINGLEBEAT*song[note][NOTEDURATION];
    int theTone=song[note][NOTETONE];
    if (theTone) {
      tone(SPEAKERPIN,theTone);
    }
    int theLed=song[note][NOTELED];
    fadeLed(theLed,HIGH,theDuration); // Fade in
    noTone(SPEAKERPIN);
    fadeLed(theLed,LOW,theDuration*PAUSEFACTOR); // Fade out + silence between note
  }
}

int playWinSequence() {
  playSong(WINSONG);
}

int playLoseSequence() {
  playSong(LOSESONG);
  Serial.println("\r\nRazzberrys!! You lost it.");
  beep(500); 
}

int getSwitchStroke() {
  while (get1stPressedSwitch()>=0) {
    // flush everything until no switch is pressed
    delay(10);
  }
  while (get1stPressedSwitch()<0) {
    // wait for next press
    delay(10);
  }
  return get1stPressedSwitch();
}

int get1stPressedSwitch() {
  for (int i=0; i<NLEDS; i++) {
    if (digitalRead(SWITCHPINS[i])==SWITCHPRESSED) {
      return i;
    }
  }
  return -1;
}

bool btnReased() {
  bool released=true;
  for (int i=0; i<NLEDS; i++) {
    if (digitalRead(SWITCHPINS[i])==SWITCHPRESSED) {
      released=false;
    }
  }
  return(released);
}

void beep(unsigned char delayms){
  byte pinState = 0;       // 0 turns it off
  for (int i=0; i<delayms; i++) {         // wait for a delayms ms
    pinState = 1 - pinState;
    digitalWrite ( SPEAKERPIN, pinState); 
    delay(1);    
  }
  delay(delayms);          // wait for a delayms ms   
}  
